# Obtained from my.telegram.org
API_ID = 2327525
API_HASH = '6d828524e7fc1a7d7331bed954ac1404'

USER_AGENT = ("Mozilla/5.0 (Windows NT 10.0; Win32; x86) AppleWebKit/537.36 (KHTML, like Gecko)"
              "Chrome/67.0.3396.99 Safari/537.36")
